
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for AutoDRM    
c type systems.
c

c
c List of subroutine in alphabetical order:
c
c   autodrm - write autodrm request file
c   autodrm_channel_select - select channels from seisnet parameter file
c   update_autodrm - update s-file after waveform transfer
c   find_autodrm_sfile - search for s-file in base that contains reference id
c   check_autodrm_file - check if file contains autodrm data
c   getdatatype - get datatype of autodrm message
c

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c  28 Feb 2003 lo: added getdatatype
c



      subroutine autodrm(station,base,email,stime,agency,parameter_file)
c
c make autodrm request file
c
      implicit none

      character*(*) parameter_file
      character*5 station,base              ! station and database name
      character*40 email                    ! email address
      character*80 line
      character*70 text
      character*19 start_time,time
      integer syear,smon,sday,shour,smin,ssec,i,j
      integer year,mon,day,doy,hour,min,sec
      real rsec
      double precision msec
      character*1 dist                       ! dist indicator
      integer length                         ! length of sfile
      character*70 sfile                     ! name od sfile
      character*80 nordicline                ! nordicline
      character*80 idline
      character*80 waveline
      logical exist                          ! file exists
      character*20 outfile
c      character*25 chan_list(6)              ! list of channels to transfer
      character*25 chan_list(6)              ! list of channels to transfer
      integer pre_time(6),time_window(6)     ! time interval
      character*42 wav_text                  ! idnumber in 42 chars
      character*14 stime                     ! start time from seisnet
      character*10 agency                    ! autodrm agency, used in msg_id
      integer nchan                          ! number of channels
      character*30 refid                     ! reference ID
      integer seiclen

C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
       integer code
C    ============= end of list ==========



  1   format(a,$)

      if (stime.eq.'NONE') then
        write(*,'(a)') 'STARTTIME  '
        write(*,'(a)') 'yyyy mm dd hh mm ss'
        read(*,'(a)') start_time

        call SEI GET VALUES(6,start_time,CODE )
         syear=ARRAY$(1)
         smon=ARRAY$(2)
         sday=ARRAY$(3)
         shour=ARRAY$(4)
         smin=ARRAY$(5)
         rsec=ARRAY$(6)

      else
        write(*,'(a)') 'start time: ',stime

        read(stime,'(i4,5i2)') syear,smon,sday,shour,smin,ssec 
      endif

      call timsec(syear,smon,sday,shour,smin,rsec,msec)

      ssec=rsec

c
c find channel list, pre event memory and time window of interest
c

      call autodrm_channel_select(station,chan_list,pre_time,
     &     time_window,nchan,parameter_file)


30    continue

c
c  make sfile name
c
      if (stime.eq.'NONE') then
        dist = 'R'
        call sfilname(syear,smon,sday,shour,
     &      smin,ssec,base,dist,sfile,length )

        inquire(file=sfile,exist=exist)

        if (exist) then
          if (ssec.le.30) ssec=ssec+2
          if (ssec.gt.30) ssec=ssec-3
          goto 30
        endif

        call file_ready(sfile)

        open(2,file=sfile,status='new')

        write(*,'(2a)') 'sfile: ',sfile

      call write_nordicline(nordicline,syear,smon,sday,shour,smin,rsec,
     &        'R',0.,0.,0.,' ','AUT',1)
        call write_idline(idline,syear,smon,sday,shour,smin,ssec)
   
        wav_text = '                                          '
        wav_text(1:14)=idline(61:74)
        wav_text(15:19)=station(1:5)
        call write_waveline(waveline,'EVE',station,'AUT',
     &     wav_text)

cNET WAV EVE MOR8 QNX 9901_04_0200_34T.MOR_03_1        STA051433 TRANSFERRED   3

        write(2,'(a80)') nordicline
        write(2,'(a80)') idline
        write(2,'(a80)') waveline
      endif

c
c produce multiple requests
c
      do j=1,nchan
        outfile='autodrm.txt '
        write(outfile(12:12),'(i1)') j

        call file_ready(outfile)
        open(1,file=outfile,status='unknown')

        text='BEGIN GSE2.0'
        write(1,'(a)') text

cMSG_TYPE REQUEST
        text='REQUEST'
        line='MSG_TYPE ' // text
        write(1,'(a)') line

cMSG_ID 

        if (stime.eq.'NONE') then      
          refid = idline(61:74)// station(1:5) // '-' // base(1:5)
        else
          refid = stime // station(1:5) // '-' // base(1:5)
        endif
        do i=1,seiclen(refid)   ! remove blanks
          if (refid(i:i).eq.' ') refid(i:i) = '_'
        enddo
        line='MSG_ID ' // refid
c
c add agency, lo 15-02-2001. at bgs
c
        line = line(1:seiclen(line))// ' ' // agency(1:seiclen(agency))

        write(1,'(a)') line

cE-MAIL seismo@ifjf.uib.no
        line='E-MAIL ' // email
        write(1,'(a)') line


cTIME 1996/12/10 10:42:00 TO 1996/12/10 10:47:00

c
c set start time
c    
        msec = msec - pre_time(j)
        call sectim(msec,year,doy,mon,day,hour,min,rsec)
        sec=rsec

        write(time,'(i4,a1,i2,a1,i2,1x,i2,a1,i2,a1,i2)')
     &  year,'/',mon,'/',day,hour,':',min,':',sec

        do i=1,19
          if (time(i:i).eq.' '.and.i.ne.11) time(i:i)='0'
        enddo
        text='TIME ' // time

        msec = msec + time_window(j)
        call sectim(msec,year,doy,mon,day,hour,min,rsec)
        sec=rsec
        write(time,'(i4,a1,i2,a1,i2,1x,i2,a1,i2,a1,i2)')
     &  year,'/',mon,'/',day,hour,':',min,':',sec

        do i=1,19
          if (time(i:i).eq.' '.and.i.ne.11) time(i:i)='0'
        enddo

        line = text(1:24) // ' TO '// time
        write(1,'(a)') line


cSTA_LIST NRA0
        line='STA_LIST ' // station
        write(1,'(a)') line


cCHAN_LIST *
c      write(*,*) 'CHAN_LIST (default = *) : '
c      read(*,'(a)') text
c
c  find channels from parameter file
c
        text=chan_list(j)
        line='CHAN_LIST ' // text
        write(1,'(a)') line


cWAVEFORM GSE2.0
        write(1,'(a)') 'WAVEFORM GSE2.0'
cSTOP
        write(1,'(a)') 'STOP'
        write(1,'(a)') 
        close(1)

      enddo

      if (stime.eq.'NONE') then

        write(2,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(2,*) ' '
        close(2)
      endif

      return
      end 


      subroutine autodrm_channel_select(station_name,chan_list,pre,
     &     length,nchan,parameter_file)
c
c read seisnet parameter file and select channels
c
      implicit none

       include 'seidim.inc'

       character*(*) chan_list(*)   ! list of channels
       integer pre(*),length(*)    ! pre event and time window length
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========

c -- station name
      character*5 station_name
c name of SEISNET parameter file
       character*(*) parameter_file
       integer read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c station index
       integer x
c parameter line
       character*80 par_line
c index
       integer ind
c station name from par file
       character*5 station
c number of channels
      integer nchan
      integer i

      chan_list(1) = '*'
      nchan = 0
      pre(1)=60
      length(1)=360
    

c
c read SEISNET paramter file
c
c      call getenv('SEISNET_PAR',parameter_file)
      write(*,*) 'SEISNET parameter file: ',parameter_file

      call file_ready(parameter_file)
            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.



      x = -1
 10   CONTINUE

      read(read01,'(a)',END=15) par_line


c   find station names
c
      if (par_line(5:12).eq.'STATION(') then
         ind=index(par_line,')')
         read(par_line(31:35),'(a5)') station
         if (station.eq.station_name) then
           read(par_line(13:ind-1),'(i3)') x
         endif

c
c  find components for station name
c
      elseif (par_line(5:14).eq.'COMPONENTS') then

        ind=index(par_line,')')

        read(par_line(16:ind-1),'(i3)') i

        if (i.eq.x) then
          nchan=nchan+1
          read(par_line(32:56),'(a25)') chan_list(nchan)

c2345678901234567890123456789012345678901234567890123456789012345678901234567890
cet COMPONENTS(26)            "*                       x           60    500  "
          read(par_line(68:72),'(i5)') pre(nchan)
          read(par_line(74:78),'(i5)') length(nchan)

        endif

      endif

      goto 10

 15   continue

c
c  close parameter file
c
      call sei close (close$,read01,code)

c
c  output
c
      if (nchan.eq.0) nchan = 1
      if (x.eq.-1) then
         write(*,*) 'station not in list !!!'
         return
      else
        write(*,*) station_name,' found ...'
        do x=1,nchan
          write(*,*) 'channels: ',chan_list(x)
          write(*,*) 'pre event memory: ',pre(x),
     &        ' window length: ',length(x)
        enddo
      endif

      return
      end



      subroutine update_autodrm(S_file,wave_autodrm,refid,autopick)
c
c update s-file after the requested waveform file has been transferred
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*8 autopick              ! name of autopick routine
      character*255 S_file              ! name of S-file
      character*40  wave_autodrm        ! name of waveform file
      character*30 refid

      logical      there                ! test if file exists

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c flag if autopick line
      logical flag
c index
      integer ind
c line with 'REQUEST'
      integer requ_line
c id
      character*14 idline
      integer seiclen


      requ_line = -1

c
c   check if file exists
c

      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file,' does not exist !'
         return
      endif

c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

      idline = '              '
c23456789012345678
c999-01-25-1009-10
      idline(1:4) = wave_autodrm(1:4)
      idline(5:6) = wave_autodrm(6:7)
      idline(7:8) = wave_autodrm(9:10)
      idline(9:10) = wave_autodrm(12:13)
      idline(11:12) = wave_autodrm(14:15)
      idline(13:14) = wave_autodrm(18:18)

c
c look for "REQUEST"
c
      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then
          if (data(x)(23:41).eq.refid(1:19)) then
            requ_line=x
          endif
        endif
      enddo


c
c  make autopick_line
c
cNET WAV EVE TRO  FTP 9809_05_0000_58T.TRO_03_1        STA050534 AUTOPICK

      if (requ_line.gt.0) then

      call write_autopick_line(data(requ_line),autopick_line,
     &    wave_autodrm,autopick,flag)

c
c  put transferred
c
c        data(requ_line)(66:76)='TRANSFERRED'
        data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo
      endif


c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo
c
c write info to new line
c
        write(waveline(2:41),'(a40)') wave_autodrm
        write(waveline(80:80),'(a1)') '6'
c
c update according to status of file transfer
c

        call file_ready(S_file)
        open(8,file=S_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo


        if (flag) write(8,'(a80)') autopick_line

        write(8,'(a80)') waveline

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c

        close(8)


      return
      end

 
 
      subroutine find_autodrm_sfile(refid,datatype)
c
c search for s-file in base that contains the reference id
c
      implicit none
      include 'seidim.inc'
      character refid*30,datatype*30,base*5
c-- start and end time of select
       character*14    start_time,end_time
c-- select key
       character*10     key
c-- see subroutine:find...
       integer          status,new_month,fstart,event_no
c-- event file name                                   
       character*80     evfile 
c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c-- number of stations
       integer          nstat
c-- number of phases
       integer nphase
c-- id line number and counters
       integer id,i,x
c read unit
       integer read01
c file exist
       logical exist
       integer code
       logical b_old
c if sfile found
       logical sfile_found
c date and time
       integer year,doy,mon,day,hour,min,isec
       real rsec
       double precision msec

C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========

      write(*,*) 'DATA_TYPE = '//datatype 
      if (datatype.eq.'ERROR_LOG'.or.
     &    datatype.eq.'LOG') then
        goto 20
      endif

c
c read basename from reference ID
c
      base=refid(21:25)
      do x=2,5
        if (base(x:x).eq.' ') base(x:x)='_' 
      enddo
      sfile_found=.false.

      key='          '

c
c start time one day before and end time one day after id time
c
      if (refid.eq.'(No REF ID received)') goto 20
      read(refid(1:14),'(i4,5i2)',err=20) year,mon,day,hour,min,isec
      rsec = isec
      call timsec(year,mon,day,hour,min,rsec,msec)
      msec = msec - 86400
      call sectim(msec,year,doy,mon,day,hour,min,rsec)
      isec=int(rsec)
      write(start_time(1:14),'(i4,5i2)') year,mon,day,hour,min,isec
      msec = msec + 86400 + 86400
      call sectim(msec,year,doy,mon,day,hour,min,rsec)
      isec=int(rsec)
      write(end_time(1:14),'(i4,5i2)') year,mon,day,hour,min,isec

      do x=1,14
        if (start_time(x:x).eq.' ') start_time(x:x)='0'
        if (end_time(x:x).eq.' ') end_time(x:x)='0'
      enddo

      write(*,*) start_time
      write(*,*) end_time

 10   continue

      CALL findevin
     * (base,start_time,end_time,key,0,
     * event_no,evfile,fstart,new_month,status)
      if(status.eq.0) then
        inquire(file=evfile,exist=exist)
        if (exist) then
            call file_ready(evfile)
            call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    evfile,                ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
         call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)
         call sei close (close$,read01,code)

         do x=1,nhead

c2345678901234567890123456789012345678901234567890123456789012345678901234567890
cNET WAV EVE NRA0 AUT 19990126115558                             REQUEST       3

            if((data(x)(66:72).eq.'REQUEST'.or.
     &         data(x)(76:78).eq.'REQ').and.
     &     data(x)(23:41).eq.refid(1:19)) 
     &        sfile_found=.true.
          enddo
        else
c          write(*,*) 'FILE DOES NOT EXIST'
        endif
c end of time
      elseif (status.eq.3) then
         goto 20
      endif

      if(sfile_found) goto 20
 
      goto 10

 20   continue
      
      if (sfile_found) then
  
         write(*,*) 'REFID: ',refid
         write(*,*) 'SFILE: '
         write(*,*) evfile
      else
         write(*,*) 'REFID NOT FOUND'
      endif

      return
      end




      subroutine check_autodrm_file(infile,autodrm_check,refid,
     &    datatype)
c
c This was originally the autodrm.f written by Urs Kradolfer. This 
c routine just checks, if a file has some BEGIN line and gets the
c reference ID
c
      implicit none
c
c 
      integer it,i,seiclen
      character refid*30,reflin*80,beglin*80,gseformat*10
      character*30 datatype
      logical startnotfound,lexist,autodrm_check
      character*60 infile

      inquire(file=infile,exist=lexist)
      if(.not.lexist)then
        goto 3000
      endif
c
c     reset/initialize some variables:
c
      startnotfound=.false.
      gseformat='GSE1.0'
c
c     open the current mail-file that has to be processed now:
c
      call FREEUNIT(it)
      open(it,file=infile,status='old')

c
c     check, if the request-mail contains a line that starts with the
c     command 'begin' or 'begin gse2.0' :
c
      call FILECHECK(it,startnotfound,gseformat,beglin)
      if(startnotfound)then
         autodrm_check=.false.
      else
         autodrm_check=.true.
      endif

      call GETREFID(it,refid,reflin)
      do i=1,seiclen(refid)
        if (refid(i:i).eq.'_') refid(i:i)=' '
      enddo
      call GETDATATYPE(it,datatype)

c     write(*,*) 'REFID ',refid

      close(it)

3000  continue

      end 


      subroutine GETDATATYPE(it,datatype)
      implicit none
      integer it, trimbeg,trimlen, i
      character datatype*30, line*80

      rewind(it)
      datatype='error_log'
      i=0
c
   1  read(it,'(a)',end=999,err=999) line
c2000      if(line.ne.' ') goto 1              ! read until end of mail-header
      call CASEFOLD(line)
      if(line(1:5).ne.'BEGIN') goto 1          ! read until start of message
c
   2  read(it,'(a)',end=999,err=999) line
      i=i+1
      call CASEFOLD(line)
c search for the 2nd data_type line
      if(line(1:9).eq.'DATA_TYPE')then
         line(1:9)=' '
         datatype=line(trimbeg(line):trimlen(line))
      endif
      if (line(1:4).eq.'WID2'.and.datatype(1:8).ne.'WAVEFORM') then
        datatype='WAVEFORM'
        goto 999
      endif
      if(i.gt.100) goto 999 ! search only in the first 100 lines for DATA_TYPE
      goto 2
c
 999  continue
      rewind(it)
c
      RETURN
      end ! of subr. getdatatype


