
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for 
c IRIS/Quanterra type stations.
c

c
c List of subroutine in alphabetical order:
c
c   iris_ch - read channel definition from seisnet parameter file
c   iris_cnordic - convert event trigger list to Nordic format
c   iris_req - write waveform data request line
c   update_iris - update s-file after waveform transfer

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c


      subroutine iris_ch
c
c read channel definition from seisnet parameter file
c

      implicit none
      include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========

c max
      integer max
      parameter (max=50)
c -- station name
      character*5 station_name
c list of channels that will be used for station
       character*6 det_channels(max)
c channel line
       character*42 channel_line(max)
c number of channels in det_channels
       integer n_ch
c name of SEISNET parameter file
       character*80 parameter_file
       integer read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c station index
       integer x
c parameter line
       character*80 par_line
c index
       integer ind
c station name from par file
       character*5 station
c -- instrument type and component
c       character*5 instrument
c number of triggered channel
c       integer channel
c -- output flag
c       logical outflag
c number of components to transfer for trigger channel
c       integer n_comp
c request time
c       character*25 req
c array of output components
c       character*5 comp_out(5)
c pre event time, request time = trigger time - pre_time
c       integer pre_time
c event lentg in seconds or samples
c       integer event_length
c time in seconds
c       double precision msec
c time
c       integer y,dy,mo,da,h,mi,isecs,yy
c       real secs
c -- time
c      integer hour, minute,sec
c      real seconds
c -- date
c      integer year,month,day
c -- waveform file line
c       character*80 waveline(max)
c counter
      integer i
      integer channel

      common /iris/det_channels,channel_line,n_ch,station_name,channel


c
c init det_channels
c

       n_ch=0
       do i=1,50
          det_channels(i) = '     '
          do x=1,42
            channel_line(i)(x:x)=' '
          enddo
       enddo

c
c read SEISNET paramter file
c
      call getenv('SEISNET_PAR',parameter_file)
      write(*,*) 'SEISNET parameter file: ',parameter_file

      call file_ready(parameter_file)
            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.



      x = -1
 10   CONTINUE

      read(read01,'(a)',END=15) par_line

c
c   find station names
c
      if (par_line(5:12).eq.'STATION(') then

         ind=index(par_line,')')

         read(par_line(31:35),'(a5)') station

         if (station.eq.station_name) then
           read(par_line(13:ind-1),'(i3)') x
         endif

c
c  find components for station name
c
      elseif (par_line(5:14).eq.'COMPONENTS') then

        ind=index(par_line,')')

        read(par_line(16:ind-1),'(i3)') i

        if (i.eq.x) then
 
          n_ch = n_ch + 1

          read(par_line(32:37),'(a6)') det_channels(n_ch)
          read(par_line(39:79),'(a41)') channel_line(n_ch)

        endif

      endif

      goto 10

 15   continue

c
c  close parameter file
c
      call sei close (close$,read01,code)

c
c  output
c
      if (x.eq.-1) then
         write(*,*) 'station not in list !!!'
         return
      else

        write(*,*) station_name,' found ...'
        write(*,*) 'channels used are: '
        do i=1,n_ch
          write(*,*) det_channels(i)
        enddo
      endif

      return
      end


      subroutine iris_req(year,month,day,hour,minute,seconds,
     &        waveline,n_comp,pre_time,event_length)
c
c write waveform data request line
c

      implicit none

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========

c max
      integer max
      parameter (max=50)
c -- station name
      character*5 station_name
c list of channels that will be used for station
       character*6 det_channels(max)
c channel line
       character*42 channel_line(max)
c number of channels in det_channels
       integer n_ch
c name of SEISNET parameter file
c       character*80 parameter_file
c       integer read01
c logical for existing file or not
c       logical          b_old
c returned code
c       integer          code
c station index
       integer x
c parameter line
c       character*80 par_line
c index
c       integer ind
c station name from par file
c       character*5 station
c -- instrument type and component
c       character*5 instrument
c number of triggered channel
       integer channel
c -- output flag
c       logical outflag
c number of components to transfer for trigger channel
       integer n_comp
c request time
       character*25 req
c array of output components
       character*6 comp_out(4)
c pre event time, request time = trigger time - pre_time
       integer pre_time
c event lentg in seconds or samples
       integer event_length
c time (day, hour, min)
       character*6 time
c time in seconds
       double precision msec
c time
       integer y,dy,mo,da,h,mi,isecs,yy
       real secs
c -- time
      integer hour, minute
c,sec
      real seconds
c -- date
      integer year,month,day
c -- waveform file line
       character*80 waveline(max)
c  counter
      integer i
c wav text
      character*42 wav_text

      common /iris/det_channels,channel_line,n_ch,station_name,channel

c
c  get components for request
c
      n_comp = 0
      do x=0,3

        if (channel_line(channel)(1+7*x:1+7*x).ne.' ') then

          comp_out(x+1)(1:6) = channel_line(channel)(1+7*x:7+7*x)
          n_comp = n_comp+1

        endif
      enddo

c
c  get preevent time in sec and event length in samples or seconds
c
cHZ   BHN   BHE               12345 60005

      if (pre_time.eq.-1) then
        read (channel_line(channel)(30:34),'(i5)') pre_time
      endif
      if (event_length.eq.-1) then
        read (channel_line(channel)(36:40),'(i5)') event_length
      endif

c
c  get time for request
      call timsec(year,month,day,hour,minute,seconds,msec)
      msec = msec - pre_time

      call sectim(msec,y,dy,mo,da,h,mi,secs)

c
c changed for Mexican station, lo 02-09-2000
c

c      if (y.ge.2000) then
c        yy=y-2000
c      else
c        yy=y-1900
c      endif

      write(req(1:4),'(i4)') y
      req(5:5)='/'
c       if (req(1:1).eq.' ') req(1:1)='0'
      write(req(6:7),'(i2)') mo
      req(8:8)='/'
        if (req(6:6).eq.' ') req(6:6)='0'
      write(req(9:10),'(i2)') da
      req(11:11)=' '
        if (req(9:9).eq.' ') req(9:9)='0'
      write(req(12:13),'(i2)') h
        if (req(12:12).eq.' ') req(12:12)='0'
      req(14:14)=':'
      write(req(15:16),'(i2)') mi
        if (req(15:15).eq.' ') req(15:15)='0'
      req(17:17)=':'
       isecs=abs(secs)
      write(req(18:19),'(i2)') isecs
        if (req(18:18).eq.' ') req(18:18)='0'

      req(20:20)=' '
 
      write(req(21:25),'(i5)') event_length
c      req(24:25) = '  '
c
c  write waveline
c

      do x=1,n_comp

        wav_text = '                                          '
        wav_text(1:6)= comp_out(x)(1:6)
        wav_text(7:31)= req(1:25)
        call write_waveline(waveline(x),'EVE',station_name,'IRI',
     &     wav_text)

  
      enddo

      return
      end


      subroutine iris_cnordic(station_n,station_type,
     &   base,lastevent_long,event_dir,wave_req_dir,det_wav_option)

c
c convert IRIS detection list to nordic format file
c
      implicit none

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========


      character*6   station_type
c max
      integer max
      parameter(max=50)
c -- station name
      character*5 station_name,station_n
c -- database name
      character*5 base
c -- time
      integer hour, minute,sec
      real seconds 
c -- date
      integer year,month,day
c -- linetype in Nordic format
c    distance indicator
      character*1 linetype, dist
c -- reporting agency
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*90 line
      character*80 nordicline,phaseline,idline
c -- index
       integer inds
c -- number of events
       integer counter
c -- output flag
       logical outflag
c -- instrument type and component
       character*12 instrument,cchannel
c -- instrument type in Nordic format
       character*1 instout
c -- instrument component
       character*1 component
c -- amplitudes
       integer signalamp, noiseamp
c -- s/n ratio
       integer snr
c -- counter
       integer i
c -- waveform file line
       character*80 waveline(max)
c time (day, hour, min)
c       character*6 time
c  instrument in waveline
       character*3 inst_wave
c number of components to transfer for trigger channel
       integer n_comp
c number of events
       integer nevent

c current and last event in database
       character*40 currentevent
c IRIS format (long), QNX format (short) last file
       character*40 lastevent_long,lastevent_short
c status to compare events
       integer stat
c list of channels that will be used for station
       character*6 det_channels(max)
c channel line
       character*42 channel_line(max)
c number of channels in det_channels
       integer n_ch
c number of triggered channel
       integer channel
c station index
       integer x
c array of output components
c       character*5 comp_out(5)
       

c read unit
c       integer read01
c name of SEISNET parameter file
c       character*80 parameter_file
c       integer read01
c logical for existing file or not
c       logical          b_old
c returned code
c       integer          code
c parameter line
c       character*80 par_line
c station name from par file
c       character*5 station
c index
c       integer ind
c pre event time, request time = trigger time - pre_time
c       integer pre_time
c event lentg in seconds or samples
c       integer event_length
c time in seconds
c       double precision msec
c time
c       integer y,dy,mo,da,h,mi,isecs,yy
c       real secs
c request time
c       character*25 req
c name of sfile
       character*80 sfile
c length of file name
       integer length
      integer seiclen
c file exist flag
      logical there
      integer pre_event,duration
c option for waveform transfer
       integer det_wav_option
c directories for network event detection and waveform transfer
       character*(*) event_dir,wave_req_dir
c last detection yyyymmddhhmmss
      character*14 lastdetection

      common /iris/det_channels,channel_line,n_ch,station_name,channel

      nevent=0 

      station_name = station_n
c
c init det_channels
c

       n_ch=0

c
c start iris_ch to get component names from seisnet parameter file
c
       call iris_ch
  
          
c
c get directory structure
c

      inds=seiclen(station_name)
      if (inds.eq.-1) inds=4


c
c open input file
c
      call file_ready(station_name(1:inds)//'.INP') 
      open(50,file=station_name(1:inds)//'.INP',status='old')

c
c set default parameters
c
      agency=station_name(1:3)
      linetype= '1'

c
c convert long format date to short format
c
      read(lastevent_long(1:4),'(a4)') lastevent_short(1:4)
      read(lastevent_long(6:7),'(a2)') lastevent_short(5:6)
      read(lastevent_long(9:10),'(a2)') lastevent_short(7:8)
      read(lastevent_long(12:13),'(a2)') lastevent_short(9:10)
      read(lastevent_long(15:16),'(a2)') lastevent_short(11:12)
      read(lastevent_long(18:19),'(a2)') lastevent_short(13:14)
      do i=1,14
         if (lastevent_short(i:i).eq.' ') lastevent_short(i:i)= '0'
      enddo
      lastdetection=lastevent_short

  20  continue
c
c read IRIS line 
c

      do i=1,90
        line(i:i)=' '
      enddo

      read(50,'(a)',end=900) line

c
c   remove ^M
c
      do i=1,90
        if (line(i:i).eq.'
') line(i:i)=' '
      enddo


      if (station_type.eq.'IRISA') then
        if ((line(21:21).ne.'=').or.(line(30:30).ne.':')) then
              goto 20
        endif
      elseif (station_type.eq.'IRISB') then
        if ((line(33:33).ne.'=').or.(line(42:42).ne.':')) then
              goto 20
        endif
      endif

      counter=counter+1

c
c  read information
c
      if (station_type.eq.'IRISA') then
c           1997 261=09/18 13:00:00.526    5513012        1000000 BHZ
        read(line(13:16),'(i4)') year
        read(line(22:23),'(i2)') month
        read(line(25:26),'(i2)') day
        read(line(28:29),'(i2)') hour
        read(line(31:32),'(i2)') minute
        read(line(34:35),'(i2)') sec
        read(line(34:37),'(f4.1)') seconds
        read(line(67:71),'(a5)') instrument
        read(line(44:50),'(i7)') signalamp 
        read(line(59:65),'(i7)') noiseamp

      elseif (station_type.eq.'IRISB') then
c        1         2         3         4         5         6         7
c2345678901234567890123456789012345678901234567890123456789012345678901234567890
channel     P A L  S/N  Year Day Date    UTC Time    Peak Amp  Period  Noise
cONO.00-LHZ D A 1 00234 1999  42=02/11 03:45:41.439         74  33.00      15

        read(line(25:28),'(i4)') year
        read(line(34:35),'(i2)') month
        read(line(37:38),'(i2)') day
        read(line(40:41),'(i2)') hour
        read(line(43:44),'(i2)') minute
        read(line(46:47),'(i2)') sec
        read(line(46:49),'(f4.1)') seconds
        read(line(1:11),'(a11)') instrument
        read(line(54:62),'(i7)') signalamp
        read(line(71:77),'(i7)') noiseamp
      endif


      write(currentevent,'(i4.4,5(i2.2))') year,month,day,hour,
     *       minute,sec

c      do i=1,14
c         if (currentevent(i:i).eq.' ') then
c           currentevent(i:i)= '0'
c         endif
c      enddo

c decide to convert or not
c if date of current if newer (greater) than date of last event
c then convert it, otherwise, bypass

      call compare(currentevent,lastevent_short,stat)
      if (stat.le.0) goto 25

c
c current event is most recent detection
c
      lastdetection=currentevent

      if (noiseamp.gt.0) then
        snr=int(signalamp/noiseamp)
      else 
        snr=-1
      endif

c
c  convert component to one character used in phaseline
c
      inst_wave= '   '

      component='?'

      if (instrument(3:4).EQ.'LP') THEN
        instout='L'
        component=instrument(1:1)
      elseif (instrument(3:5).EQ.'VBB') THEN
        instout='B'
      elseif (instrument(1:2).eq.'LH') THEN
        instout='L'
        component=instrument(3:3)
      elseif (instrument(1:2).eq.'BH') THEN
        instout='B'
        component=instrument(3:3)
      elseif (instrument(1:2).eq.'BL') THEN
        instout='B'
        component=instrument(3:3)
      elseif (instrument(1:2).eq.'SH') THEN
        instout='S'
        component=instrument(3:3)
      endif

      dist= 'R'

c
c  check if component will be used as trigger, channel will be used in iris_req
c
      if (station_type.eq.'IRISA') then
        do i=1,n_ch
          if (instrument.eq.det_channels(i)) then
            channel=i
            outflag=.TRUE.
          endif
        enddo

      elseif (station_type.eq.'IRISB') then
        do i=1,n_ch

          if (station_n(4:4).ne.' ') then
            cchannel = station_n(1:4) // '.' // det_channels(i) 
          else
            cchannel = station_n(1:3) // '.' // det_channels(i)
          endif

          if (instrument.eq.cchannel) then
            channel=i
            outflag=.TRUE.
          endif
        enddo
      endif
      
      if (outflag.EQV..TRUE.) THEN

c
c write nordic line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') month
      write(nordicline(9:10),'(i2)') day   
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') minute
      write(nordicline(17:20),'(f4.1)') seconds
      write(nordicline(22:22),'(a1)') dist
c      write(nordicline(45:45),'(a1)') '*'
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(a1)') '1'
      write(nordicline(80:80),'(a1)') linetype


      sec=int(seconds)

c
c  write ID line
c
      call write_idline(idline,year,month,day,hour,minute,sec)

c xxx
      pre_event=-1
      duration=-1
      call iris_req(year,month,day,hour,minute,seconds,
     &        waveline,n_comp,pre_event,duration)
c
c  write phase line
c
      phaseline=' '
      write(phaseline(2:6),'(a5)') station_name
      write(phaseline(7:7),'(a1)') instout
      write(phaseline(8:8),'(a1)') component
      write(phaseline(10:10),'(a1)') 'E'
      write(phaseline(16:16),'(a1)') 'A'
      write(phaseline(19:20),'(i2)') hour
      write(phaseline(21:22),'(i2)') minute
      write(phaseline(24:28),'(f5.2)') seconds
      if (snr.le.9999) write(phaseline(57:60),'(i4)') snr
c
c output
c

555   continue

      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )


      inquire(file=sfile,exist=there)
      if (there) then
        if (sec.le.30) then
          sec = sec + 1
        else
          sec = sec - 1
        endif
      endif

      call file_ready(sfile)
      open(60,file=sfile,status='new')


      write(*,*) nordicline
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      do x=1,n_comp
        write(60,'(a80)') waveline(x)
      enddo
      write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
c        write(60,'(a80)') phaseline
        write(60,*)

      nevent= nevent+1
      close(60)

      call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

      endif

      outflag=.FALSE.

  25  continue
      goto 20

 900  continue

      close(50)

      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',lastdetection,' ...'

      return
      end


 
      subroutine update_iris(S_file,date,time,wave_name,
     &   station_name,channel,autopick)
c
c routine to update s-file after the waveform file has been transferred
c

      implicit none
 
      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'
 
      character*8 autopick              ! name of autopick routine
      character*255 S_file        ! name of S-file
      character*5 station_name    ! name of station 
      logical      there             ! test if file exists
      character*6  channel           ! component
      character*40  wave_name    ! waveform filename
      character*8   time         ! time in waveform request line
      character*10  date
 
c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c flag if autopick line
      logical flag
c index
      integer ind
      integer seiclen

 
 
 
c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file,' does not exist !'
         return
      endif
 
 
c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')
 
c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)
 
      close(8)
 
      requ_line=-1
c
c look for "REQUEST"
c
cNET WAV EVE KONO IRI 10-BHZ99/02/10 02:10:03  6000              REQUEST       3
cNET WAV EVE CUIG IRI BHZ   2000/02/06 11:52:02  6000            REQUEST       3

      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then
         if (data(x)(14:17).eq.station_name(1:4)) then
          if (data(x)(23:28).eq.channel(1:6)) then      ! check component
           if (data(x)(29:38).eq.date(1:10)) then
            if (data(x)(40:47).eq.time(1:8)) then
 
                  requ_line=x

            endif
           endif
          endif
         endif
        endif
      enddo
 
      if (requ_line.eq.-1) then
         write(*,*) 'no request in S-file ... !'
         return
      endif
 

c
c  make autopick line
c
      if (wave_name.ne.'never')
     &  call write_autopick_line(data(requ_line),autopick_line,
     &    wave_name,autopick,flag)
 
c
c update line
c
c        data(requ_line)(66:76) = 'TRANSFERRED'
      if (wave_name.ne.'never') then
        data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo
 
c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo

c
c write info to new line
c
         write(waveline(2:41),'(a40)') wave_name
         write(*,*) wave_name 
         write(waveline(80:80),'(a1)') '6'
      else
        data(requ_line)(76:78) = 'NEV'   ! changed April 2001, lo
      endif

c
c update according to status of file transfer
c
 
 
 
        call file_ready(S_file)
        open(8,file=S_file,status='unknown')
 
        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo
 
        if (wave_name.ne.'never') then
          if (flag) write(8,'(a80)') autopick_line
          write(8,'(a80)') waveline
        endif
 
        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c
 
        close(8)
 
 
      return
 
      end
 
