
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa
c

c This file contains SEISNET FORTRAN subroutines that are used for FTP server
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   quake_cnordic - convert finger quake output to Nordic file format

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c



      subroutine quake_cnordic(station_name,base,lastevent,region,cbase,
     &    ext_stat,ext_chan,ext_duration,parameter_file,wav_req_dir)
c
c convert output from: finger quake@gldfs.cr.usgs.gov
c to nordic file format
c

      implicit none
      include 'seidim.inc'
      include 'libsei.inc'

      character*(*) parameter_file
      character*(*) wav_req_dir
c -- station name
      character*5 station_name 
c  database name
      character*5 base
c -- name of central database
      character*5 cbase
c -- time
      integer hour, minute, sec
c -- region
      character*(*) region
c -- text string of stations to extract
      character*(*) ext_stat
c -- text string giving channels, seperated by &
      character*(*) ext_chan
c -- time duration for extract
      real ext_duration
c -- pre event time in seconds
      real ext_pre
c -- coordinates of region
      integer min_lat,max_lat,min_long,max_long
c -- max magnitude of interest
      real max_mag
c -- date
      integer year,month,day
c -- date and time of last event
      integer last_year,last_month,last_day
      integer last_hour,last_minute,last_sec
c -- location
      real latitude, longitude
c -- depth, magnitude, seconds
      real depth, magnitude, seconds 
c -- lat and lon for (E/W), (N/S)
c    linetype in Nordic format
c    distance indicator
      character*1 lat, lon, linetype, dist
      character*2 magtype
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*80 line,nordicline,idline,infoline
c -- top-directory
       character*60 top_directory
c -- index
       integer ind,inds,pind
c---directory separator character
      character*1 dchar
c -- text strings
       character*80 chr_text_in,chr_text_out
       character*240 sfile
c -- counter
       integer i,j,x,y,a
c -- put events for given region directly into the database?
       logical into_central_base
c -- yes if event in given region
       logical in_region
c
       character*40 lastevent ! last event in data base
       character*40 currentevent ! current event in data base
       integer      stat     ! flag to compare date of events
c number of events
       integer nevent
c length of file name, counter
      integer length,c,code
      integer seiclen
      logical ext_flag
      integer ext_stat_array(50),ext_stat_n
      integer ext_chan_n(100)
      character*10 ext_chan_array(100,10)
      character*80 chan_array(100)
      character*14 lastdetection
      
      nevent=0
      ext_flag=.false.
      ext_stat_n=0
      ext_pre=90.

c 
c check name of database
c
      do c=2,5
        if (cbase(c:c).eq.' ') cbase(c:c)='_'
        if (base(c:c).eq.' ') base(c:c)='_'
      enddo
c
c get directory structure
c

      call topdir(top_directory)

      ind=seiclen(top_directory)
      inds=seiclen(station_name)
      if (inds.eq.-1) inds=5

      call dir_char(dchar)         ! dirctory delimiter character
c
      chr_text_in = top_directory(:ind)//
     &           dchar // 'WOR' //dchar                 //
     &           station_name(1:inds) // '.INP'

      chr_text_out = top_directory(:ind)//
     &           dchar // 'WOR' //dchar                 //
     &           station_name(1:inds) // '.OUT'

c
c check if extract is wanted
c
      if (ext_stat(1:2).ne.'NO'.and.ext_stat(1:2).ne.'no') then
        ext_flag=.true.
c
c read in all the stations
c
        ext_stat_n = 0
        do i=1,seiclen(ext_stat)
          if (ext_stat(i:i).eq.'_') then
            ext_stat(i:i)=' '
            ext_stat_n = ext_stat_n + 1
          endif
        enddo
        ext_stat_n = ext_stat_n + 1
        call SEI GET VALUES(ext_stat_n,ext_stat,CODE )
        do i=1,ext_stat_n
          ext_stat_array(i)=int(ARRAY$(i))
c          write(*,*) i,ext_stat_array(i)
        enddo
      endif
c
c read in channels for extract
c
      if (ext_chan(1:2).ne.'NO'.and.ext_chan(1:2).ne.'no') then
        do i=1,100
          ext_chan_n(i) = 1
          ext_chan_array(i,1)='NONE'
        enddo
        j=1
        x=1
        do i=1,seiclen(ext_chan)
          if (ext_chan(i:i).eq.'&') then
            chan_array(j)=ext_chan(x:i-1)
            j=j+1
            x=i+1
          endif
        enddo 
        chan_array(j)=ext_chan(x:seiclen(ext_chan))   ! last element

        do x=1,j
          i=1 
          a=1
          do y=1,seiclen(chan_array(x))
            if (chan_array(x)(y:y).eq.'@') then
              if (i.eq.1) then
                read(chan_array(x)(1:y-1),'(i)') ind
                i=y+1
              else
                ext_chan_array(ind,a)=chan_array(x)(i:y-1)
                i=y+1
                a=a+1
              endif
            endif
          enddo
          ext_chan_array(ind,a)=chan_array(x)(i:seiclen(chan_array(x)))
          ext_chan_n(ind)=a
        enddo

      endif
        
c
c open files
c
      ind=seiclen(chr_text_in)

      call file_ready(station_name(1:inds)//'.INP')

      open(50,file=station_name(1:inds)//'.INP',status='old')
      ind=seiclen(chr_text_out)
c     open(60,file=chr_text_out,status='undefined')
c      open(60,file=station_name(1:inds)//'.OUT',status='undefined')

c
c set default parameters
c
      agency=station_name(1:3)
      linetype= '1'

c get date of last event
      read(lastevent(1:4),'(i4)') last_year
      read(lastevent(5:6),'(i2)') last_month
      read(lastevent(7:8),'(i2)') last_day
      read(lastevent(9:10),'(i2)') last_hour
      read(lastevent(11:12),'(i2)') last_minute
      read(lastevent(13:14),'(i2)') last_sec
      lastdetection=lastevent
c     write(*,'(a8,1x,i4,5(i2))') 'last   :',last_year,last_month,
c    *last_day,last_hour,last_minute,last_sec

c
c  get region coordinates
c
      if (region.ne.'no') then

        ind=index(region,',') -1
        read(region(1:ind),'(i4)') min_lat
        pind=ind+2

        ind=index(region(pind:),',') -1
        read(region(pind:pind+ind),'(i4)') max_lat
        pind=pind+ind+1

        ind=index(region(pind:),',') -1
        read(region(pind:pind+ind),'(i4)') min_long
        pind=pind+ind+1

        ind=index(region(pind:),',') -1
        read(region(pind:pind+ind),'(i4)') max_long
        pind=pind+ind+1

        ind=index(region(pind:),' ') -1
        read(region(pind:pind+ind),'(f)') max_mag

        into_central_base = .TRUE.

      endif
 

c
c find first epicentre line
c
c  10  continue
c      read(50,'(a80)',end=900) line
c      if (line(1:8).eq.'yy/mm/dd') goto 20
c      goto 10

  20  continue

c
c read QUAKE line 
c
      read(50,'(a80)',end=900) line

        write(*,*) line
c      if (line(3:3).ne.'/') goto 20
      if (line(3:3).ne.'/'.and.line(2:5).ne.'PDE-') goto 20
      if (line(1:2).eq.'yy') goto 20

      do i=1,80
         if (line(i:i).eq.'
') line(i:i)=' ' 
         infoline(i:i)=' '
      enddo

      infoline(2:79)=line(1:79)
      infoline(80:80)='3'

      if (line(2:5).ne.'PDE-') then

        read(line(1:2),'(i2)') year
        if (year.lt.50) then
          year=year+2000
        else
          year=year+1900
        endif
        read(line(4:5),'(i2)') month
        read(line(7:8),'(i2)') day
        read(line(10:11),'(i2)') hour
        read(line(13:14),'(i2)') minute
        read(line(16:17),'(i2)') sec
        seconds= sec * 1.0
        read(line(20:24),'(f5.2)') latitude
        read(line(25:25),'(a1)') lat
        read(line(27:32),'(f6.2)') longitude
        read(line(33:33),'(a1)') lon
        read(line(35:39),'(f5.1)') depth
        read(line(41:43),'(f3.1)') magnitude
        read(line(44:45),'(a2)') magtype
      else
c
c read lines of type: USGS/NEIC (PDE) 1973 - Present 
c                     Screen File Format (80 columns) 
c Example:
cPDE-W 2002   12 01 075601.22 -16.08 -172.96  33 6.00 MwGS    ...G .......
c
        read(line,'(7x,i4,3x,i2,1x,i2,1x,i2,i2,f5.2,1x,
     &           f6.2,1x,f7.2,1x,i3,1x,f3.1,2x,a2)')
     &    year,month,day,hour,minute,seconds,latitude,longitude,depth,
     &    magnitude,magtype
        lat='E' 
        lon='N' 
      endif
      write(currentevent,'(i4,5(i2))') year,month,day,hour,
     *minute,sec
      do i=1,14
         if (currentevent(i:i).eq.' ') then
           currentevent(i:i)= '0'
         endif
      enddo

c
c decide to convert or not
c if date of current if newer (greater) than date of last event
c then convert it, otherwise, bypass
c
      call compare(currentevent,lastevent,stat)
c     write(*,'(a8,1x,i4,5(i2))') 'current:',year,month,day,hour,
c    *minute,sec
      if (stat.le.0) goto 35
      lastdetection=currentevent

  25  continue
c     write(*,*) 'CONVERTING ...'

c      year= year + 1900

c
c     convert to nordic format
c
      if (lat.eq.'S') latitude= latitude * -1.0
      if (lon.eq.'W') longitude= longitude * -1.0
      dist= 'D'
      if (magtype(2:2).eq.'b')  magtype(2:2)= 'B'
      if (magtype(2:2).eq.'w')  magtype(2:2)= 'W'
      if (magtype(2:2).eq.'l')  magtype(2:2)= 'L'
      if (magtype(2:2).eq.'s')  magtype(2:2)= 'S'
      if (magtype(2:2).eq.'d')  magtype(2:2)= 'D'

c
c  check if event in given region
c
      in_region = .false.
      if (into_central_base.eqv..TRUE.) then
       
        if (latitude.le.max_lat.and.latitude.ge.min_lat.and.
     &      longitude.le.max_long.and.longitude.ge.min_long.and.
     &      magnitude.ge.max_mag)
     &     in_region = .true.
 
c        if (in_region) write(*,*) line
 
      endif

      
c
c write nordic line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') month
      write(nordicline(9:10),'(i2)') day   
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') minute
      write(nordicline(17:20),'(f4.1)') seconds
      write(nordicline(22:22),'(a1)') dist
      write(nordicline(24:30),'(f7.3)') latitude
      write(nordicline(31:38),'(f8.3)') longitude
      write(nordicline(39:43),'(f5.1)') depth
      write(nordicline(45:45),'(a1)') '*'
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(a1)') '1'
      write(nordicline(73:75),'(f3.1)') magnitude
      write(nordicline(76:76),'(a1)') magtype(2:2)
      write(nordicline(77:79),'(a3)') agency
      write(nordicline(80:80),'(a1)') linetype

c
c make idline
c

      call write_idline(idline,year,month,day,hour,minute,sec)

c
c output
c
      sfile = ' '

      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )

 
      call file_ready(sfile) 
      open(60,file=sfile,status='unknown')

      write(*,*) nordicline
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      write(60,'(a80)') infoline
      write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'

      write(60,*) 

      nevent=nevent+1

      close(60)

c
c   if events should be written to central database directly in case they
c   are within given region
c

      if (in_region) then

        call sfilname(year,month,day,hour,
     &      minute,sec,cbase,dist,sfile,length )


        call file_ready(sfile)
        open(70,file=sfile,status='unknown')

c
c  write 2 headerlines
c
        write(70,'(a80)') nordicline
        write(70,'(a80)') nordicline

        write(70,'(a80)') idline
        write(70,'(a80)') infoline
      write(70,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(70,*)

        close(70)

c
c if extract call add_data to add request
c
        if (ext_flag) then
          do i=1,ext_stat_n
            if (ext_chan_n(ext_stat_array(i)).eq.0) then
              ext_chan_n(ext_stat_array(i))=1
              ext_chan_array(ext_stat_array(i),1)='*'
            endif
            do j=1,ext_chan_n(ext_stat_array(i))
c
c call routine to add request
c
c         write(*,*) ' Duration ',ext_duration
              call add_data(parameter_file,cbase,sfile,
     & ext_stat_array(i),ext_chan_array(ext_stat_array(i),j),
     & ' ',ext_duration,ext_pre,'NONE')
c     & ' ',ext_duration,ext_pre,wav_req_dir)
c            write(*,*) i,j,ext_chan_array(ext_stat_array(i),j)
            enddo
          enddo
          if (wav_req_dir.ne.'NONE') then
            write(*,*) ' calling wave_request ... '
            call wave_request(sfile,wav_req_dir)
          endif
        endif
      endif



  35  continue
      goto 20

 900  continue

      close(50)
      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',lastdetection,' ...'
      
      return
      end



