c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c changes
c
c  17 June 2001 lo: bug fix, close station file, if station not found
c

      subroutine calc_min_tt(station_name,c,deqlat,deqlon,depth,tmin,
     &                  dist,iout)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c subroutine to calculate minimum travel time for given station
c and hypocenter location, local and teleseismic distances
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c input:
c         station_name - 5 character station code
c         c            - model indicator
c         deqlat       - epicenter latitude in deg
c         deqlon       - epicenter longitude in deg
c         depth        - hypocenter depth
c
c output:
c         tmin         - minimum travel time
c         dist         - distance between epicenter and station
c         iout         - 0 if error, otherwise 1
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c      implicit none
      save

      include 'hypparm.inc'

      character*80 infile
      character*12 stfile
      character*80 indat,testdat
      character*5 station_name
      character*1 reff,ucase,prmd,prm2
      character*4 stat
      character*8 phsid
      integer dlat,dlon,height,iustat,j,i,nmoho,n
      integer nconrad,nl,nn,nd,iulst,iflag,minflag
      real mlat,mlon,pi,rearth,degtorad,ztr,delta
      real xnear,xfar,depth,dist,pos,tmin,ann,testj
      character*1 clat,clon,c
      logical station_found
c latitude and longitude of station and epicenter in radians
      real slat,slon,eqlat,eqlon,sslat,sslon
c input of eq lat and lon in degrees
      real deqlat,deqlon
c distance from earthqauke to station in degree
      real dedeg
c         az0      azimuth from earthquake to station measured clockwise
c                     from north in degrees
      real azi,baz
c output
      integer iout
c file exist
      logical exists
c top directory
      character*60 top_directory
c dlimin...
      character*1 dchar
      external sei clen
      integer  sei clen


      iout=0
c
c check for station_name
c
      if (station_name.eq.'     ') return

      pi=3.141593
      degtorad=pi/180.
      rearth=6371.

c 
c   get directory structure
c
      call topdir(top_directory)
      call dir_char(dchar)         ! dirctory delimiter character

c
c  set name of station file
c
      if(c.eq.' '.or.c.eq.'0')then
        stfile='STATION0.HYP'
      else
        stfile='STATION'  // c // '.HYP'
      endif
c
c open input file
c
      infile=stfile

      iustat=9         
      inquire(file=infile,exist=exists)
      if (exists) then
        open(iustat,status='old',file=infile)
        write(*,*) ' station file:',infile
      else
c
c check in DAT directory
c
        infile = top_directory(:seiclen(top_directory)) //
     &           dchar // 'DAT' //dchar                 //
     &           stfile
        inquire(file=infile,exist=exists)
        if (exists) then
          open(iustat,status='old',file=infile)
          write(*,*) ' station file:',infile
        else
          write(*,*) infile,' does not exist'
          return
        endif
      endif

c
c convert input into radians
c
      eqlat=deqlat*pi/180
      eqlon=deqlon*pi/180
c      call fold_lat(eqlat)
c      deqlat=eqlat*180/pi
      

c
c    set test parameter defaults
c
      call settest(test)

c    read test parameter changes and station data
c    changed 4/94 to read character string and check for ")" so reset test(1) is OK
        stat='xxxxx'
        j=-1
        
        do while (stat.ne. '    '.and.j.ne.0)
         read(iustat,'(a80)')testdat
         if(testdat(14:14).eq.')')then
          read(testdat,'(a4,t12,i2,t16,f9.4)')stat,j,testj
         elseif(testdat(13:13).eq.')')then
          read(testdat,'(a4,t12,i1,t15,f9.4)')stat,j,testj
         else
          read(testdat,'(a4)')stat
          j=-1
         endif
         if(j.gt.0)then
          test(j)=testj
c          if(iulst.gt.0)write(iulst,'(1x,''Reset test('',i2,'')='',
c     &    f10.4)')j,testj
         endif
        end do

c
c read station coordinates
c
      indat='XXXXXXXX'
      station_found=.false.
      do while (indat(1:8).ne.'        ')
        read(iustat,'(a80)')indat
        if (indat(2:6).eq.station_name) then
           read(indat(7:27),'(i2,f5.2,a1,i3,f5.2,a1,i4)')
     &           dlat,mlat,clat,dlon,mlon,clon,height
           station_found=.true.
        elseif (indat(3:6).eq.station_name) then
           read(indat(7:27),'(i2,f5.2,a1,i3,f5.2,a1,i4)')
     &           dlat,mlat,clat,dlon,mlon,clon,height
           station_found=.true.
        endif
      end do

      if (.not.station_found) then
         write(*,*) 'station not in list'
         close(iustat)
         return
      else
c
c  convert coordinates
c
      slat = dlat + real(mlat)/60. 
      if(clat .eq. 'S') slat = -slat
      slon = dlon + real(mlon)/60. 
      if(clon .eq. 'W') slon = -slon

         sslat=slat
         sslon=slon

         slat=slat*pi/180
         slon=slon*pi/180

         call delaz(slat,slon,dist,dedeg,az0,eqlat,eqlon)
         delta=dedeg
         write(*,*) ' station found'
      endif

c read the velocity model
        i=1
        iustat=9
        
c    reff is used to specify the moho layer for PN calculation
5       read(iustat,105,end=99)v(i),d(i),vs(i),reff
105     format(3f7.3,a1)
        if(ucase(reff).eq.'N')nmoho=i

c 4/94: added nconrad variable
        if(ucase(reff).eq.'B')nconrad=i
        
        if(v(i).eq.0.0)go to 6
        i=i+1
        go to 5

c    nl is the number of layers in the velocity model
6       nl=i-1

      
c    read in trial depth and vp/vs ratio
        read(iustat,'(3f5.0,f5.2)',end=99)ztr,xnear,xfar,pos

c    if vs(1)=0 then set vs(i)=v(i)/pos
        if(vs(1).eq.0.0)then
          do i=1,nl
            vs(i)=v(i)/pos
          end do
        endif

c  store thicknesses in parm
        do i=1,nl-1
          parm(nl+i)=d(i+1)-d(i)
        end do

        do i=1,nl
          parm(i)=v(i)
        end do
        nn=2*nl-1

      minflag=int(test(63))      
        
c    xs(1), xs(2) are the station long. and lat.
      x0(1)=0.0
      x0(2)=0.0
      x0(3)=0.0
      
      prmd=' '                        
      prm2=' '


      prmd=' '
      iulst=10
      nd=100

      xh(2)=0.0
      xh(3)=depth

       do i=1,nl
         parm(i)=v(i)
       end do

       xh(1)=dist/rearth
c
c  calculate travel time
c

      if (dist.le.test(57)) then

c
c  local 
c

        write(*,*) 'local event'
        call dtdx2(nn,parm,xh,x0,prmd,nmoho,nconrad,iulst,tmin,
     &  dx,delta,ann,iflag,phsid)
      else

c
c teleseismic distance
c
        write(*,*) 'teleseismic event'

c
c  convert to geocentric coordinates
c
        call fold_lat(sslat)
        call fold_lat(deqlat)

c        write(*,*) sslat,sslon,deqlat,deqlon
        call distaz(sslat,sslon,deqlat,deqlon,delta,azi,baz)

c         write(*,*) 'delta: ' , delta
         call ttim(delta,depth,tmin)
      endif   ! of teleseismic

      if (tmin.gt.0) iout=1

      
99    continue
      close(9)

      return
      end

      




c****************************************************************
c Modified version of Ray Buland's global travel time generation
c program which runs on both PC's and SUN's
c must be linked with subroutines in tau.for

c reads the two random access files iasp91.hed & iasp91.tbl
c

      subroutine ttim (delta,depth,tmin)
      save
      real delta,depth,zs
      parameter (max=100)
      logical log,prnt(3)
      character*8 phcd(max),phlst(10)
      character*80 fnam
      character*80 modnam
      dimension tt(max),dtdd(max),dtdh(max),dddp(max),mn(max),ts(max)
      dimension xcor(max),tcor(max),usrc(2)


      logical exist
      character*1 dchar
      character*60 top_directory

c  min tt in seconds  

      real tmin
      data inrec/30/,phlst(1)/'query'/,prnt(3)/.true./
 
      tmin = -1
c      modnam='IASP91 '
      call iasp91_filename(modnam)
      modnam = modnam(:index(modnam,' ')-1) // '.TBL'
      call dir_char(dchar)
      call topdir(top_directory)
      itp= index(top_directory,' ') -1
c
c   check for iaspei files
c
      inquire(file=modnam,exist=exist)
      if(exist) then
         modnam = modnam(:index(modnam,'.TBL')-1)
      else
         modnam=
     *    top_directory(1:itp)//dchar//'DAT'//dchar//
     *    modnam(:index(modnam,'.TBL')-1)
      endif

      fnam='ttim1.lis'//char(0)
      call assign(10,2,fnam)
      call tabin(inrec,modnam)
      prnt(1)=.false.
      prnt(2)=.false.
      phlst(1)='basic   '

c
      zs=depth
      if(zs.lt.0.) go to 999
      call depset(zs,usrc)

c      write(*,*) 'delta ',delta
      call trtm(delta,max,n,tt,dtdd,dtdh,dddp,phcd)
      if(n.le.0) go to 98

      tmin = tt(1)

c      do i=1,n
c        write(*,102)i,phcd(i),tt(i),mn(i),ts(i),dtdd(i),dtdh(i),dddp(i)
c 102    format(7x,i5,2x,a,f9.2,i4,f7.2,1p3e11.2)
c
c      enddo 

      goto 999

 98   continue

      write(*,101) delta
 101  format(/1x,'No arrivals for delta =',f7.2)

 999  continue

      return
      end



cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine fold_lat(alat)
c
c-------- given geographic lat compute geocentric lat
c
c input:  la     degree portion of latitude in degrees
c         ins    n for north, s for south
c         ala    minutes portion of latitude
c         lo     degree portion of longitude
c         iew    e for east, w for west
c         alo    minutes portion of longitude
c output: alat   geocentric latitude in radians
c         alon   longitude in radians
      parameter (pi = 3.14159265)
      parameter (twopi = 2.0*pi)
      parameter (halfpi = 0.5*pi)
      parameter (rad = pi/180.)
      parameter (deg = 1.0/rad)
      parameter (equrad = 6378.2064)
      parameter (polrad = 6356.5838)
      parameter (flat = (equrad - polrad)/equrad)
      parameter (c1 = (1.0 - flat)**2)
      parameter (c2 = halfpi*(1.0/c1 - 1.0))
c
c
      alat = alat*rad
c  convert from geographic to geocentric latitude
      if (halfpi-abs(alat) .ge. 0.02) goto 201
         alat = alat/c1-sign(c2,alat)
         goto 202
  201    alat = atan(c1*tan(alat))
  202    continue
         alat=alat/rad
      return
      end


      subroutine XXXdistaz (alat1,alon1,alat2,alon2, delta,azi,baz)
c
c Calculate angular distance, azimuth and backazimuth between two
c  points on a sphere.
c
c Input
c
c   ALAT1,ALON1  =  latitude and longitude of point 1.
c   ALAT2,ALON2  =  latitude and longitude of point 2.
c
c Output
c
c   DELTA  =  angular distance between points 1 and 2.
c   AZI    =  azimuth from north of point 2 w.r.t. point 1.
c   BAZ    =  azimuth from north of point 1 w.r.t. point 2.
c
c All arguments are in degrees.
c Latitude, longitude and DELTA are geocentric.
c Latitude is zero at equator and positive north.
c Longitude is positive toward the east.
c AZI and BAZ are positive and measured clockwise from local north.
c
c      implicit double precision (a-h,o-z)
c     real alat1,alat2,alon1,alon2,delta,azi,baz
      save
      pi=3.14159265
      rtod=180./pi
      dtor=pi/180.
c
      rlat1 = dtor*alat1
      rlat2 = dtor*alat2
      rdlon = dtor*(alon2-alon1)
c
      clat1 = cos(rlat1)
      clat2 = cos(rlat2)
      slat1 = sin(rlat1)
      slat2 = sin(rlat2)
      cdlon = cos(rdlon)
      sdlon = sin(rdlon)
c
      cdel = slat1 * slat2 + clat1 * clat2 * cdlon
      cdel = min(cdel,1.0)
      cdel = max(cdel,-1.0)
      yazi = sdlon * clat2
      xazi = clat1 * slat2 - slat1 * clat2 * cdlon
      ybaz = -sdlon * clat1
      xbaz = clat2 * slat1 - slat2 * clat1 * cdlon
c
      delta = rtod * acos(cdel)
c     write(16,*)'delta ',delta
c      write(*,*)' xazi yazi ',xazi,yazi
      if(yazi.ne.0.0.or.xazi.ne.0.0)then
        azi = rtod * atan2(yazi,xazi)
        baz = rtod * atan2(ybaz,xbaz)
      else
        azi=0.0
        baz=0.0
      endif 
      if (azi.lt.0.) azi=azi+360.
      if (baz.lt.0.) baz=baz+360.
c
      return 
      end


