
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c    program:    event.for
c    by:         Lars Ottemoeller
c
c program to collect events from different databases to declare events.
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c updates:  
c
C      31.07.97
c      19.01.98   LO    use end_time
c      28.03.98   lo    correct number of networks in header line
c      31.08.98   lo    bug due to increasing the maximum number of
c                       events, central database gets lost, fixed
c      31.08.98   lo    made event.inc
c      02.09.98   lo    corrected dimensions of arrays, start with 0
c
c ---- Seisnet 1.2  ----  for Seisan 7.0    
c
c      21.12.98   lo    changed and tested
c      08.04.99   lo    include last trigger for single trigger
c      10.06.99   lo    remove crash if seconds is '**' 
c      25.06.99   lo    put error protection into date_to_sec
c      29.10.99   lo    use i4 in date_to_sec
c      02.11.99   lo    fix id line in make_file
c      18.05.00   lo    keep event type when merge into central base
c      03.10.02   lo    fix id for new cdb events
c      02.12.03   lo    moved check for det force to main program
c


c ---- main program --------

       program event 

       implicit none

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========


       include 'event.inc'

c write unit  #1,2
       integer          write02
c read unit  #1
       integer          read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c start and end time of select     
       character*14    start_time,end_time
c data base name if not agency (blank)
       character*40     base_name
       integer seiclen
c name of SEISNET parameter file
       character*80 parameter_file
c database names
       character*5 database_name(max_database)
c is database used
       logical database_active(max_database)
c flag to force event detection also if only single trigger
       logical force_det(max_database)
c counter
       integer c,x,y,z
c text line
       character*80 line
c index
       integer ind
c number of days that are processed
       integer ndays
c time interval of event
       integer event_window
c mininmum number of triggers read from parameter file
       integer min_trig 
c number of events read from the central database
       integer counter_cdb
c saving name of central database
       character*5 cdb_save
       logical exist
c waveform request directory
       character*240 wave_req_dir
c
        integer number_of_events 

c list of S-files for one event, (x,y), x is event index, y gives the number
c of the S-file, y=0 is the number of elements
        integer events(0:max_events,0:max_database)

      common /event_block/ events

c
c version
c
      call print_ver 
     & ('SEISNET - event - version: 2.0.5   date: 06/04/2000')

      scounter=0

c
c read SEISNET paramter file
c
      call getenv('SEISNET_PAR',parameter_file)
      write(*,*) 'SEISNET parameter file: ',parameter_file

      do c=1,max_database
        database_name(c)= ' '
        database_active(c)=.FALSE.
        force_det(c)=.false.
      enddo

         call file_ready(parameter_file)
            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

      c=0
      ndays = 1     !  standard value is one day

 10   CONTINUE

      read(read01,'(a)',END=20) line

c
c   find database names
c
      if (line(5:12).eq.'DATABASE') then

         ind=index(line,')')
       
         read(line(14:ind-1),'(i3)') x
         read(line(31:35),'(a5)') database_name(x)

         do z=2,5 
           if (database_name(x)(z:z).eq.' ') database_name(x)(z:z)='_'
         enddo
      elseif (line(5:28).eq.'EVENTDETECTION_DIRECTORY') then
c
c look for eventdetection directory
c
        read(line(31:seiclen(line)),'(a)') eventdet_dir

        inquire(file=eventdet_dir,exist=exist)                         
        if (.not.exist) then
          write(*,*) ' no such directory: '//eventdet_dir
          stop
        endif
        write(*,*) 'Eventdetection directory: ',eventdet_dir     

      elseif (line(5:29).eq.'WAVEFORMREQUEST_DIRECTORY') then
c
c look for waveform request directory
c
        read(line(31:seiclen(line)),'(a)') wave_req_dir

        inquire(file=wave_req_dir,exist=exist)
        if (.not.exist) then
          write(*,*) ' no such directory: '//wave_req_dir
          stop
        endif
        write(*,*) 'Waveform request directory: ',wave_req_dir

c
c   find out if database should be used for event detection
c
      elseif (line(5:18).eq.'EVENTDETECTION') then
         if (line(31:33).eq.'YES') then
           ind=index(line,')') 
           read(line(20:ind-1),'(i3)') x
           database_active(x)=.TRUE.
         endif

c
c   read the number of processing days
c
      elseif (line(5:18).eq.'PROCESSINGDAYS') then
         read(line(31:33),'(i3)') ndays

c
c   read length of event time window
c
      elseif (line(5:19).eq.'EVENTTIMEWINDOW') then
         read(line(31:35),'(i5)') event_window

c
c   read minimum number of triggers
c
      elseif (line(5:19).eq.'MINIMUMTRIGGERS') then
        read(line(31:35),'(i5)') min_trig    
        write(*,*) 'min ',min_trig

      elseif (line(5:18).eq.'FORCEDETECTION') then
        if (line(31:33).eq.'YES') then
          ind=index(line,')')
          read(line(20:ind-1),'(i3)') x
          write(*,*) ' forcing detection for network: ',x
          force_det(x)=.true.
        endif

c
c   read name of central database
c
      elseif (line(5:19).eq.'CENTRALDATABASE') then
        read(line(31:35),'(a5)') central_database

        do z=2,5
          if (central_database(z:z).eq.' ') central_database(z:z)='_'
        enddo

        write(*,*) 'Central database: ',central_database
        cdb_save = central_database

      endif


      c=c+1

      goto 10

 20   CONTINUE
      
c
c   output which databases will be used
c
      do x=1,max_database
        if (database_active(x)) then 
          write(*,*) database_name(x)
        endif
      enddo

c
c   get time interval, starting ndays before now
c
       call GetDateBefore(ndays, start_time,end_time)
       write(*,*) 'start time : ' , start_time
       start_time(9:14)='      '
       write(*,*) 'start time : ' , start_time



 50   continue 


c
c   collect S-files from the central database first 
c
    
      base_name = central_database
      call collect (base_name,start_time,end_time,.false.)

      counter_cdb = scounter 

c
c   collect S-files from detection directory
c
      base_name=',,'              

c      call collect (base_name,start_time,end_time,force_det(x))
      call collect (base_name,start_time,end_time,.false.)

c
c   open output event file
c
          call file_ready('list.out')
            call sei open(unknown$+warn$,        ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'list.out',            ! File name
     &                    write02,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
 

c
c output
c

      write(*,*) 'total number of S-files : ',scounter
c  
c    get times in secs
c
      call event_date_to_sec

c
c    sort events, order will be given in pi(x)
c
      call event_sort

c
c set forced detections
c
      do x=1,scounter
        par_force(pi(x))=.false.
        do y=1,max_database
          if (database_name(y).eq.par_network(pi(x)).and.
     &         force_det(y).eq..true.) then
            par_force(pi(x))=.true.
          endif
        enddo
      enddo

c
c  write sorted list of events to list.out
c
      do x=1,scounter
 
        write(write02,'(i4,1x,i4,1x,a5,1x,a17,1x,a)') x,pi(x)
     & ,par_network(pi(x)),par_time(pi(x)),par_sfile(pi(x))
 
      enddo
 
c
c  find events, events given in events variable
c
      call find_events (event_window,events,number_of_events,
     &    min_trig,force_det)


c
c   give number of S-file that are events
c
      write(write02,*) ' '
      write(write02,*) 'Events '


        do x=1,number_of_events
          write(write02,'(i4,1x,i4)')
     &            x,events(x,0)
          do y=1,events(x,0)
            write(write02,'(i4,1x,i4)')
     &            y
          enddo
          write(write02,*)
        enddo

      call sei close (close$,write02,code)

c
c  make events, merge S-files
c
      central_database = cdb_save
      write(*,*) 'now making events...'
      call make_events(min_trig,counter_cdb
     & ,number_of_events,wave_req_dir)

      stop
      end

cccccccccc    END OF MAIN PROGRAM   ccccccccccc



cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c

      subroutine collect (base_name,start_time,end_time,force)

c
c this subroutine is a simplified version of the collect program
c
c Lars Ottemoeller, 23.07.97
c

       implicit none
       include 'seidim.inc'

       include 'event.inc'


C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========
C
c-- event data
       character*80	data(max_data)	
c-- exp indicator                                         
       character*1	exp		
c-- event type                                      
       character*1      type		
c-- number of recors for event                           
       integer		nrecord		
c-- number of header lines                                 
       integer		nhead		
c-- number of stations                                     
       integer		nstat		
c-- number of phases
       integer nphase
c-- start and end time of select      
       character*14    start_time,end_time 
c-- data base name if not agency (blank)               
       character*40	base_name	
c-- event file name                                   
       character*80	evfile		
c-- select key                                       
       character*10     key		
c-- see subroutine:find...       
       integer		status,new_month,fstart,event_no 
c-- true if file exists (CLK)             
       logical          exist           
c-- id line number
       integer id
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c read unit #1
       integer          read01
       integer x                              ! counter
c set par_force to true if force is true, to force detection
       logical force
 
c                                                                               
c  start read and write loop                                                    
c                                                                               
 5     continue                                                                 
c-- always use next event                                
       key='          '		
       CALL findevin                                                       
     * (base_name,start_time,end_time,key,0,
     * event_no,evfile,fstart,new_month,status)                                 
       if(status.eq.0) then                                                     
c--   (CLK)     
         inquire(file=evfile,exist=exist)                         
c--   (CLK)     
         if (exist) then                                          
            call file_ready(evfile)
            call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    evfile,                ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
         call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

c
c   check if event already associated, if yes then skip this event
c

         do x=1,nhead

           if (data(x)(2:12).eq.'ONL ASO YES'.OR.
     &         data(x)(2:12).eq.'NET ASO YES') then

c             write(6,'(1x,a80,a)') evfile(1:80),'already associated'
             
             call sei close (close$,read01,code)
             goto 5
           endif
         enddo

         scounter=scounter+1
 
c
c  read information from first line into arrays
c
          par_sfile(scounter) = evfile(1:80)
          par_time(scounter) = data(1)(2:18)
c       write(*,*) ' xxx ',data(1)(2:18),par_sfile(scounter)
          if (par_time(scounter)(1:2).eq.'  ') then
            par_time(scounter)(1:2) = '19'
          endif
c not used yet
c          par_dist(scounter) = type

c
c changed lot 12-07-2002
c          par_network(scounter) = base_name(1:5)
          par_network(scounter) = data(1)(46:48)
          if (base_name(1:5).eq.central_database) then
            par_network(scounter) = base_name(1:5)
          endif

c          if (force) then
c            write(*,*) ' xxx forcing ... ',scounter,evfile(1:80)
c            par_force(scounter)=.true.
c          else
c            par_force(scounter)=.false.
c          endif

c
c not used yet
c 
c          read(data(1)(24:30),'(f)') par_latitude(scounter)
c          read(data(1)(31:38),'(f)') par_longitude(scounter)



c                                                                               
c  print event name                                                             
c                                                                               
c           write(6,'(1x,a80)') evfile(1:80)
           call sei close (close$,read01,code)
c-- (CLK)
         else                                                      
c-- (CLK)      
           write(6,'(1x,a80,a)') evfile(1:80),' doesn''t exist.'         
c-- (CLK)      
         endif                                                     
c-- back for next event                                         
         goto 5			
       else                                                                     
c-- 3 is end of time period                       
          if(status.eq.2) write(6,*)' End of index file'
          if(status.gt.3)       
     *    write(6,*)' ****** Something wrong, status= ', status                 
       endif                                                                    

      return
      end

c -----   end of subroutine collect   ---------
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


                                                                                
                                                                                

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine event_sort

c     by Lars Ottemoeller, 22.07.97
c
c
c
c
c  this subroutine is made to sort events for time, the order in the 
c  array is not changed, the sorted order will be given in an index
c  array, pi(x)
c
c  the method used is very simple, the array is scanned as often as
c  there are elements in the array, the smallest will be found, and the
c  corresponding number stored in the index array
c


       implicit none

       include 'event.inc'


c counter
      integer x,y
c smallest time string
      integer smallest
c array of already sorted time
      logical sorted(max_parameter)
      double precision time_num 

      do x=1,max_parameter
         sorted(x)=.FALSE.
      enddo

      smallest=scounter

      do x=1,scounter

c        time_string='99999999999999999'  
 
         time_num=9E12

        do y=1,scounter
        
c          if ((par_time(y).lt.time_string).AND.

          if ((par_sec(y).lt.time_num).AND.
     & (sorted(y).eqv..FALSE.)) then
      
c            time_string = par_time(y)

            time_num = par_sec(y)
            smallest = y

          endif


        enddo

        sorted(smallest)=.TRUE.
        pi(x) = smallest

      enddo


      return
      end            ! end of subroutine event_sort

c   end of event_sort
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc






cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine GetDateBefore(n,datebef,curr)

c Luis A. Arriola: jul 1997
c
c Calculate date for n days before curr date
c

        implicit none

        integer year, month, day, dofy, hour, min,secs
        real sec
        double precision msec       !   time in seconds
        character*14 curr, datebef
        integer c                      ! counter
        integer n


c
c   get system time
c
        call gmttim(curr)

c
c  read time from current time
c
        read(curr(1:4),'(i4)') year
        read(curr(5:6),'(i2)') month
        read(curr(7:8),'(i2)') day
        read(curr(9:10),'(i2)') hour
        read(curr(11:12),'(i2)') min
        read(curr(13:14),'(i2)') secs

        sec = secs    ! sec real in timesec

c
c   convert time to seconds
c
        call timsec(year,month,day,hour,min,sec,msec)

c
c  subtract n days in secs
c
        msec= msec - real(n * 86400.0)

c
c  convert seconds back to time
c
        call sectim(msec,year,dofy,month,day,hour,min,sec)

        secs = abs(sec)

c
c   output
c
        write(*,*) n,' days before ...'
        write(datebef,'(i4,i2,i2,i2,i2,i2)') year,month,day,hour,min
     &  ,secs

        do c=1,14 
          if (datebef(c:c).eq.' ') datebef(c:c)='0'
        enddo


        return
      end

c     end of subroutime GetDateBefore
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine event_date_to_sec

c
c read timestring as given from S-files and get the number of seconds
c
c  by Lars Ottemoeller , july 97

       implicit none

        integer year, month, day,  hour, min, secs
        real sec
        double precision msec       !   time in seconds
        integer c                      ! counter
	integer seiclen


       include 'event.inc'

        do c=1,scounter

c
c   read the time
c
          read(par_time(c)(1:4),'(i4)',err=4100) year
          read(par_time(c)(6:7),'(i2)',err=4100) month
          read(par_time(c)(8:9),'(i2)',err=4100) day 
          read(par_time(c)(11:12),'(i2)',err=4100) hour
          read(par_time(c)(13:14),'(i2)',err=4100) min
          read(par_time(c)(16:17),'(i2)',err=4001) secs
          goto 4002
4001      secs = 0
4002      continue

          sec = secs

c
c convert date and time to seconds
c
          call timsec(year,month,day,hour,min,sec,msec)
	  goto 5000

4100      continue
          write(*,*) ' error: ',par_sfile(c)(1:seiclen(par_sfile(c)))
          msec=0.
5000      continue
          par_sec(c) = msec
        enddo



        return 
    
      end

ccccccccc   end of date_to_sec
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine find_events (event_window,events,
     &  number_of_events,min_trig)

c
c  by Lars Ottemoeller , july 97
 
       implicit none

       include 'event.inc'

       double precision last_time     !   time in seconds
       integer c,x,y,z                ! counter
       integer event_window
       integer last_event             ! last declared event

c list of S-files for one event, (x,y), x is event index, y gives the number
c of the S-file, y=0 is the number of elements
        integer events(0:max_events,0:max_database)
        integer x_events(max_database)
 
        integer number_of_events

c counter 
        integer xcounter,ycounter
c 
       logical new_event,cdb

       integer min_trig    !  minimum number of triggers

 
        integer base_number                     ! number of different bases
        character*5 base_test(max_database)     ! database names
        integer base_counter(max_database)      ! number of events from base 
        logical new_base                        ! new base
        integer n_single                        ! number of single events
        integer single_events(max_parameter)    ! array of single events
        logical dup_flag                        ! indicate that 2 detections from same network

c
c  init
c

        if (scounter.eq.0) return

        xcounter = 0

        do x=0,max_events
          do y=0,max_database
            events(x,y) = 0
          enddo
        enddo

        c = 2
        if(scounter.eq.1) c=1
        last_time = par_sec(pi(1))
        last_event = 1
        ycounter=0
 
        new_event = .true.       

  10    continue

c 
c check if network of detection is already associated with current detection
c
        dup_flag=.false.
        do x=1,ycounter
         if (par_network(pi(c)).eq.par_network(events(xcounter,x)))then
            dup_flag=.true.
         endif
        enddo

c
c  check if current event close enough to last event in time and not both events
c  not from central database
c
        if ((par_sec(pi(c)).le.par_sec(pi(last_event))+event_window).
     &    and..not.(ycounter.eq.0.and.par_network(pi(c)).eq.
     &    central_database.and.par_network(pi(last_event)).eq.
     &    central_database).and..not.dup_flag)
     &    then

c          write(*,'(a,i4,1x,i4,f8.1)') 'EVENT ',pi(last_event),pi(c),
c     &     par_sec(pi(c))-par_sec(pi(last_event)) 


          if (new_event) then
             new_event = .false.
c             cdb=.false.
             xcounter=xcounter+1
             ycounter=1
             events(xcounter,ycounter) = pi(last_event)
          endif
          ycounter = ycounter + 1
          events(xcounter,ycounter) = pi(c)
          events(xcounter,0) = ycounter

        else

          new_event = .true.

c
c  sinlge trigger only if not from central database
c
          if (min_trig.eq.1.or.par_force(pi(last_event))) then
            if (ycounter.eq.0) then
              if (par_network(pi(last_event)).ne.central_database) then

c         write(*,*) ' force ',pi(last_event),par_force(pi(last_event))
                xcounter = xcounter + 1
                events(xcounter,1) = pi(last_event)
                events(xcounter,0) = 1

              endif
            endif
          endif
  
          last_event = c
          ycounter = 0

        endif

        c = c + 1

        if (c.GT.scounter) goto 20

        goto 10

  20    continue

c
c check if last trigger is single trigger
c
          if (min_trig.eq.1.or.par_force(pi(c-1))) then
c          if (min_trig.eq.1) then
            if (ycounter.eq.0) then
              if (par_network(pi(c-1)).ne.central_database) then
                xcounter = xcounter + 1
                events(xcounter,1) = pi(c-1)
                events(xcounter,0) = 1
              endif
            endif
          endif            

        number_of_events = xcounter       

c
c set number of sinlgle events that have to be added later
c
        n_single=0
        do x=1,max_parameter
           single_events(x)=0
        enddo

        do x=1,number_of_events
c
c  get number of detections from the same database
c

          base_number = 0

          do z=1,max_database
            base_test(z) = ' '
            base_counter(z) = 0
          enddo

          do y=1,events(x,0)
 
            new_base = .TRUE.
            c=base_number
            do z=1,c

              if (events(x,y).ne.-100) then
               if (base_test(z).eq.par_network(events(x,y))) then
  
                  base_counter(z)=base_counter(z)+1
                  new_base = .FALSE.
c
c mark element in events to delete later, number set to -100
c
                  if (central_database.ne.
     &                  par_network(events(x,y))) then
                    n_single=n_single+1
                    single_events(n_single)=events(x,y)
                  endif
c                  write(*,*) n_single,' ',events(x,y)
                  events(x,y) = -100

               endif  
              endif
            enddo
            if (new_base) then
              base_number=base_number + 1
              base_test(base_number)=par_network(events(x,y))
              base_counter(base_number)=base_counter(base_number)+1
            endif 
          enddo

c         do z=1,base_number
c             write(*,'(i3,1x,a3,1x,i3)') z,base_test(z),base_counter(z)
c         enddo

c
c  clean event
c

          do y=1,max_database
            x_events(y)=0
          enddo

          c=0
          do y=1,events(x,0)

            if (events(x,y).gt.-1) then
              c=c+1
              x_events(c)=events(x,y)
            endif
          
          enddo

          do y=0,max_database
            events(x,y)=0
          enddo
              
          events(x,0) = c

          do y=1,events(x,0)
            events(x,y) = x_events(y)
          enddo
        enddo

c
c   add the single events if not from central database
c

        do z=1,n_single

          if (par_network(single_events(z)).ne.central_database)
     &       then
          
            number_of_events=number_of_events+1
          
            events(number_of_events,0) = 1
            events(number_of_events,1) = single_events(z)

c          write(*,*) number_of_events,events(number_of_events,0),
c     &      events(number_of_events,1) 
          endif

        enddo

c         write(*,*) '2 - number of events : ',number_of_events


        return
  
      end
c                end of subroutine find_events            
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc





cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c

      subroutine make_events(min_trig,counter_cdb,
     &   number_of_events,wave_req_dir)


c  subroutine gets the index of S-files that are one event, for these
c  S files a line is written to the original S-file that the file has
c  been associated, then the S-files are written to one event and written
c  to one file for all events, that can be split
c
c  by Lars Ottemoeller , july 97
 
       implicit none

c minimum number of triggers
       integer min_trig
c waveform request directory
       character*(*) wave_req_dir

 
       include 'event.inc'

c number of s-files in the central database
       integer counter_cdb 

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
      integer seiclen
C
C    ============= end of list ==========
 
       integer x,y,z,i,j              ! counter

c list of S-files for one event, (x,y), x is event index, y gives the number
c of the S-file, y=0 is the number of elements
        integer events(0:max_events,0:max_database)
        integer number_of_events 
        integer counter
c output file
        integer write01,read01,write02,write03 
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c-- event data
       character*80     data1(max_data),data2(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord1,nrecord2
c-- number of header lines
       integer          nhead1,nhead2
c-- number of stations
       integer          nstat
c-- number of phases
       integer nphase
c-- id line number
       integer id
c-- id text
       character*14 id_text
c number of ASO line
       integer aso_line
c line
       character*80 line
c event lines
       character*80 event(max_database,max_data)
c       character*80 event_sorted(max_data)
c       integer xhead(max_database)
c       integer xphase(max_database)
c event line index
       integer index
c  flag if events found
       logical flag
c number of event in the central database
       integer cdb_event
c number
       integer num
c file exist
c       logical exist
c date and time
       integer year,month,day,hour,minute,sec
c sfile name
       character*80 sfile
c
       integer length
c
       character*1 dist

      common /event_block/ events



        flag = .FALSE.
        aso_line=0
        do i=1,80 
          line(i:i)=' '
        enddo
        line(80:80) = '3'
        line(2:12) = 'NET ASO YES'

         do x=1,number_of_events

c  counter has the number of triggers
            counter = events(x,0)
c
c   check if one of the events in the central database and set variable
c   this event will be directly written to the central database, to the
c   specific file
c
           cdb_event = 0
           do y=1,counter
             if (events(x,y).le.counter_cdb) then
                cdb_event = events(x,y)
             endif
           enddo

c
c   check if number of triggers larger than the minimum
c
c          write(*,*) ' xxx ',x,' 1 ',par_force(events(x,1))
           if (counter.ge.min_trig.or.
     &         par_force(events(x,1))) then

c
c   init
c
            do z=1,counter       
              do i=1,max_data 
                event(z,i) = ' '
              enddo
            enddo

             write(*,'(a,i3)') 'event number : ',x

             do z=1,counter

               write(*,'(a80)') par_sfile(events(x,z))
c
c   open S-file to read
c

 
            call file_ready (par_sfile(events(x,z)))

            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    par_sfile(events(x,z)),! File name
     &                    read01,                ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

c 
c  read S-file
c

         call indata(read01,nstat,nphase,nhead2,nrecord2,type,exp,
     &               data2,id)

         call sei close (close$, read01, code)

c
c copy S-file to event array, for first event 
c
            if (z.eq.1) then

               do i=1,nrecord2
                  data1(i) = data2(i)
               enddo
               nhead1=nhead2
               nrecord1=nrecord2

c
c  otherwise merge the files
c
            elseif (z.gt.1) then
              call merge_f(data1,data2,nhead1,nhead2,nrecord1,nrecord2) 
            endif

c
c change ID line type back
c
             do i=1,nhead2
               if (data2(i)(2:8).eq.'ACTION:') then
                  data2(i)(80:80) = 'I'
c                  write(*,*) data2(i)(1:80)
               endif
             enddo

 
c
c   only for events not in the central data base the ASO line is added
c
             if (events(x,z).gt.counter_cdb) then

c 
c check if NET ASO line exists
c
c               do i=1,nhead2
c
c                 if (data2(i)(2:8).eq.'ONL ASO'.OR.
c     &               data2(i)(2:8).eq.'NET ASO') then
c
c                    aso_line = i
c
c                 endif
c               enddo

c
c   open S-file file to write
c
            call file_ready(par_sfile(events(x,z)))
            call sei open(unknown$+warn$,        ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    par_sfile(events(x,z)),     ! File name
     &                    write02,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
               call sei close (delete$, write02, code)
              endif
             enddo

c
c add line to event in station database, data2, will not be read again
c
c               if (aso_line.eq.0) then    ! no ASO line
c
c                 write(write02,'(a)') (data2(i),i=1,nhead2-1)
c                 write(write02,'(a)') line         ! NET ASO YES
c                 write(write02,'(a)') data2(nhead2)
c                 write(write02,'(a)') (data2(i),i=nhead2+1,nrecord2)
c               else
c
c                 data2(aso_line) = line
c                 write(write02,'(a)') (data2(i),i=1,nrecord2)
c               endif

c               call sei close (close$, write02, code)

             index=0 
c
c  clear character 45 in first line to enable location
c
             if(counter.gt.1) data1(1)(45:45)= ' '

c  write type 7 line
c             event_sorted(index)= event(1,xhead(1))


c
c  if this event is not in the CDB, write event to database       
c
              
             if (cdb_event.eq.0) then 
c
c open output file
c 
          read(data1(1)(2:5),'(i4)') year
          read(data1(1)(7:8),'(i2)') month
          read(data1(1)(9:10),'(i2)') day
          read(data1(1)(12:13),'(i2)') hour
          read(data1(1)(14:15),'(i2)') minute
          read(data1(1)(17:18),'(i2)') sec

          read(data1(1)(22:22),'(a1)') dist

          call sfilname(year,month,day,hour,
     &      minute,sec,central_database,dist,sfile,length)

            call file_ready(sfile)

            call sei open(unknown$+warn$,        ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    sfile,                 ! File name
     &                    write01,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

c
c fix ID
c
               do i=1,nhead1
                 if (data1(i)(80:80).eq.'I') then
                   write(data1(i)(61:74),'(i4.4,5i2.2)') 
     &                year,month,day,hour,minute,sec
                   write(*,*) data1(i)(61:74)
                 endif
               enddo

c
c  write number of stations to first line
c
               write(data1(1)(49:51),'(i3)') counter
               write(write01,'(a)') (data1(i),i=1,nrecord1)
               write(write01,*)
               call sei close (close$, write01, code)

c
c crate waveform requests
c
               call wave_request(sfile,wave_req_dir)
             else

c
c  write number of stations to first line
c
 
               num = 0
               read(data1(1)(49:51),'(i3)') num
               num=events(x,0)+num-1
               write(data1(1)(49:51),'(i3)') num

c
c make sure event type in header is the same as in file name
c
               data1(1)(22:22) = par_sfile(cdb_event)
     *            (seiclen(par_sfile(cdb_event))-8:
     *             seiclen(par_sfile(cdb_event))-8)

c
c  change ID-line to action new
c
               do i=1,nhead1    
                 if (data1(i)(80:80).eq.'I') then
                   data1(i)(9:11) = 'NEW' 
                 endif
               enddo
c
c   if one of the S-files of this event is in cdb then write to this file
c
               call file_ready(par_sfile(cdb_event))
               call sei open(unknown$+warn$,     ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    par_sfile(cdb_event),  ! File name of CDB file
     &                    write03,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
c
c check that ID number agrees with filename
c
               j=seiclen(par_sfile(cdb_event))
               id_text='              '
               do i=1,nhead1
                 if (data1(i)(80:80).eq.'I') then
                   id_text = par_sfile(cdb_event)(j-5:j-2) //
     &                       par_sfile(cdb_event)(j-1:j) //       
     &                       par_sfile(cdb_event)(j-18:j-17) //       
     &                       par_sfile(cdb_event)(j-15:j-12) //       
     &                       par_sfile(cdb_event)(j-10:j-9) 
                    write(data1(i)(61:74),'(a14)') id_text
                 endif
               enddo
 
               write(write03,'(a)') (data1(i),i=1,nrecord1)
               write(write03,*)
               call sei close(close$, write03, code)
c
c create waveform requests
c
               call wave_request(par_sfile(cdb_event),wave_req_dir)
             endif
           endif
         enddo

         return
       end

c    end of make_event
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      
c      subroutine systim(t)
c      implicit none
c
c      character*14 t
c      integer hr,min,sec,year,mon,day,sec100
c      integer time(3),date(3),i
c      logical sun,pc,linux
c      call computer_type(sun,pc,linux)
cc
cc   get system time on sun
c
c       if(sun.or.linux) then
c         call itime(time)
c         call idate(date)
c         write(t,'(i4,5i2)') 
c     *   date(3),date(2),date(1),time(1),time(2),time(3)
c       endif
cc
cc   check for blanks
cc
c       do i=1,14
c          if(t(i:i).eq.' ') t(i:i)='0'
c       enddo
c       
c      return
c      end	  	  	  
c



